#include <stdio.h>
#include <math.h>

#include "graph.h"

extern "C" {
  void TV4(double *,int,int,double,double);
  void TV8(double *,int,int,double,double);
}
// TV-minimization 
// this can be linked with C code (tested only on linux) provided the
// flag -lstdc++ is given to the linker
// TV4 = with nearest neighbours interaction
// TV8 = with also next nearest

void TV4(double *G, // solution is returned in the same array
	 int sx,int sy,// size of image
	 double lambda, // weight on the total variation
	 double erreur)  // erreur pour le calcul du min

{
  typedef Graph<double,double,double> GraphType;
  int i,j,k,sxy=sx*sy;
  double dval, dr,drd;

  // GraphType::node_id * nodes, no;

  if (sx <=2 || sy <= 2)
    { fprintf(stderr,"error: bad size\n"); exit(0); }

  lambda *= 0.78539816339744830962 ; // pi/4
  // renormalization to ensure that the TV
  // is "as close as possible" to the isotropic TV [TV4 is anyway very far]

  dr=lambda;

  GraphType *BKG = new GraphType(sx*sy,4*sx*sy);

  for (i=0;i<sxy;i++) {
    BKG->add_node();
    // BKG->add_tweights(i,G[i],0.); // could use
    BKG->set_trcap(i,G[i]);
  }
  
  for (i=1;i<sx;i++) BKG->add_edge(i-1,i,dr,dr);
  for (j=1,k=sx;j<sy;j++) {
    BKG->add_edge(k-sx,k,dr,dr);
    for (i=1,k++;i<sx;i++,k++) {
      BKG->add_edge(k-1,k,dr,dr);
      BKG->add_edge(k-sx,k,dr,dr);
    }
  }

  BKG->graphTV(erreur);
  //BKG->maxflow(); // for testing
  //if (BKG->error()) { fprintf(stderr,"error in maxflow\n"); exit(0); }
  //if (BKG->error()) { fprintf(stderr,"error in maxflow\n"); exit(0); }
  
  for (k=0;k<sxy;k++) G[k]=BKG->get_trcap(k);
  delete BKG;
}

void TV8(double *G, // solution is returned in the same array
	 int sx,int sy,// size of image
	 double lambda, // weight on the total variation
	 double erreur)  // erreur pour le calcul du min

{
  typedef Graph<double,double,double> GraphType;
  int i,j,k,sxy=sx*sy;
  double dval, dr,drd;

  // GraphType::node_id * nodes, no;

  if (sx <=2 || sy <= 2)
    { fprintf(stderr,"error: bad size\n"); exit(0); }

  lambda *= 0.78539816339744830962 ; // pi/4
  // renormalization to ensure that the TV
  // is "as close as possible" to the isotropic TV [TV4 is anyway very far]

  dr=lambda;
  drd=lambda*0.70710678118654752440;
  
  GraphType *BKG = new GraphType(sx*sy,4*sx*sy);

  for (i=0;i<sxy;i++) {
    BKG->add_node();
    // BKG->add_tweights(i,G[i],0.); // could use
    BKG->set_trcap(i,G[i]);
  }
  
  for (i=1;i<sx;i++) BKG->add_edge(i-1,i,dr,dr);
  for (j=1,k=sx;j<sy;j++) {
    BKG->add_edge(k-sx,k,dr,dr);
    for (i=1,k++;i<sx;i++,k++) {
      BKG->add_edge(k-1,k,dr,dr);
      BKG->add_edge(k-sx,k,dr,dr);
      BKG->add_edge(k-1,k-sx,drd,drd);
      BKG->add_edge(k-sx-1,k,drd,drd);
    }
  }
  
  BKG->graphTV(erreur);
  //BKG->maxflow(); // for testing
  //if (BKG->error()) { fprintf(stderr,"error in maxflow\n"); exit(0); }
  //if (BKG->error()) { fprintf(stderr,"error in maxflow\n"); exit(0); }
  
  for (k=0;k<sxy;k++) G[k]=BKG->get_trcap(k);
  delete BKG;
}
